"""
Mapping from VT100 (ANSI) escape sequences to the corresponding keys.
"""
from ..events import Key, Mods, KeyPressEvent

NO_MODS           = Mods.NO_MODS
ALT               = Mods(True , False, False)
CONTROL           = Mods(False, True , False)
SHIFT             = Mods(False, False, True )
ALT_CONTROL       = Mods(True , True , False)
ALT_SHIFT         = Mods(True , False, True )
CONTROL_SHIFT     = Mods(False, True , True )
ALT_CONTROL_SHIFT = Mods(True , True , True )

ANSI_ESCAPES = {
    "\x00": KeyPressEvent(" ", CONTROL),
    "\x01": KeyPressEvent("a", CONTROL),
    "\x02": KeyPressEvent("b", CONTROL),
    "\x03": KeyPressEvent("c", CONTROL),
    "\x04": KeyPressEvent("d", CONTROL),
    "\x05": KeyPressEvent("e", CONTROL),
    "\x06": KeyPressEvent("f", CONTROL),
    "\x07": KeyPressEvent("g", CONTROL),
    "\x08": KeyPressEvent("h", CONTROL),
    "\x09": KeyPressEvent(Key.Tab, NO_MODS),
    "\x0a": KeyPressEvent("j", CONTROL),
    "\x0b": KeyPressEvent("k", CONTROL),
    "\x0c": KeyPressEvent("l", CONTROL),
    "\x0d": KeyPressEvent(Key.Enter, NO_MODS),
    "\x0e": KeyPressEvent("n", CONTROL),
    "\x0f": KeyPressEvent("o", CONTROL),
    "\x10": KeyPressEvent("p", CONTROL),
    "\x11": KeyPressEvent("q", CONTROL),
    "\x12": KeyPressEvent("r", CONTROL),
    "\x13": KeyPressEvent("s", CONTROL),
    "\x14": KeyPressEvent("t", CONTROL),
    "\x15": KeyPressEvent("u", CONTROL),
    "\x16": KeyPressEvent("v", CONTROL),
    "\x17": KeyPressEvent("w", CONTROL),
    "\x18": KeyPressEvent("x", CONTROL),
    "\x19": KeyPressEvent("y", CONTROL),
    "\x1a": KeyPressEvent("z", CONTROL),
    "\x1b": KeyPressEvent(Key.Escape, NO_MODS),
    "\x9b": KeyPressEvent(Key.Escape, SHIFT),
    "\x1c": KeyPressEvent("\\", CONTROL),
    "\x1d": KeyPressEvent("]", CONTROL),
    "\x1e": KeyPressEvent("^", CONTROL),
    "\x1f": KeyPressEvent("-", CONTROL),
    "\x7f": KeyPressEvent("h", CONTROL),

    "\x1b[1~": KeyPressEvent(Key.Home, NO_MODS),
    "\x1b[2~": KeyPressEvent(Key.Insert, NO_MODS),
    "\x1b[3~": KeyPressEvent(Key.Delete, NO_MODS),
    "\x1b[4~": KeyPressEvent(Key.End, NO_MODS),
    "\x1b[5~": KeyPressEvent(Key.PageUp, NO_MODS),
    "\x1b[6~": KeyPressEvent(Key.PageDown, NO_MODS),
    "\x1b[7~": KeyPressEvent(Key.Home, NO_MODS),
    "\x1b[8~": KeyPressEvent(Key.End, NO_MODS),
    "\x1b[Z": KeyPressEvent(Key.Tab, SHIFT),
    "\x1b\x09": KeyPressEvent(Key.Tab, SHIFT),
    "\x1b[~": KeyPressEvent(Key.Tab, SHIFT),

    "\x1bOP": KeyPressEvent(Key.F1, NO_MODS),
    "\x1bOQ": KeyPressEvent(Key.F2, NO_MODS),
    "\x1bOR": KeyPressEvent(Key.F3, NO_MODS),
    "\x1bOS": KeyPressEvent(Key.F4, NO_MODS),
    "\x1b[[A": KeyPressEvent(Key.F1, NO_MODS),
    "\x1b[[B": KeyPressEvent(Key.F2, NO_MODS),
    "\x1b[[C": KeyPressEvent(Key.F3, NO_MODS),
    "\x1b[[D": KeyPressEvent(Key.F4, NO_MODS),
    "\x1b[[E": KeyPressEvent(Key.F5, NO_MODS),
    "\x1b[11~": KeyPressEvent(Key.F1, NO_MODS),
    "\x1b[12~": KeyPressEvent(Key.F2, NO_MODS),
    "\x1b[13~": KeyPressEvent(Key.F3, NO_MODS),
    "\x1b[14~": KeyPressEvent(Key.F4, NO_MODS),
    "\x1b[15~": KeyPressEvent(Key.F5, NO_MODS),
    "\x1b[17~": KeyPressEvent(Key.F6, NO_MODS),
    "\x1b[18~": KeyPressEvent(Key.F7, NO_MODS),
    "\x1b[19~": KeyPressEvent(Key.F8, NO_MODS),
    "\x1b[20~": KeyPressEvent(Key.F9, NO_MODS),
    "\x1b[21~": KeyPressEvent(Key.F10, NO_MODS),
    "\x1b[23~": KeyPressEvent(Key.F11, NO_MODS),
    "\x1b[24~": KeyPressEvent(Key.F12, NO_MODS),
    "\x1b[25~": KeyPressEvent(Key.F13, NO_MODS),
    "\x1b[26~": KeyPressEvent(Key.F14, NO_MODS),
    "\x1b[28~": KeyPressEvent(Key.F15, NO_MODS),
    "\x1b[29~": KeyPressEvent(Key.F16, NO_MODS),
    "\x1b[31~": KeyPressEvent(Key.F17, NO_MODS),
    "\x1b[32~": KeyPressEvent(Key.F18, NO_MODS),
    "\x1b[33~": KeyPressEvent(Key.F19, NO_MODS),
    "\x1b[34~": KeyPressEvent(Key.F20, NO_MODS),
    "\x1b[1;2P": KeyPressEvent(Key.F13, NO_MODS),
    "\x1b[1;2Q": KeyPressEvent(Key.F14, NO_MODS),
    '\x1b[1;2R': KeyPressEvent(Key.F15, NO_MODS),
    "\x1b[1;2S": KeyPressEvent(Key.F16, NO_MODS),
    "\x1b[15;2~": KeyPressEvent(Key.F17, NO_MODS),
    "\x1b[17;2~": KeyPressEvent(Key.F18, NO_MODS),
    "\x1b[18;2~": KeyPressEvent(Key.F19, NO_MODS),
    "\x1b[19;2~": KeyPressEvent(Key.F20, NO_MODS),
    "\x1b[20;2~": KeyPressEvent(Key.F21, NO_MODS),
    "\x1b[21;2~": KeyPressEvent(Key.F22, NO_MODS),
    "\x1b[23;2~": KeyPressEvent(Key.F23, NO_MODS),
    "\x1b[24;2~": KeyPressEvent(Key.F24, NO_MODS),
    "\x1b[1;5P": KeyPressEvent(Key.F1, CONTROL),
    "\x1b[1;5Q": KeyPressEvent(Key.F2, CONTROL),
    "\x1b[1;5R": KeyPressEvent(Key.F3, CONTROL),
    "\x1b[1;5S": KeyPressEvent(Key.F4, CONTROL),
    "\x1b[15;5~": KeyPressEvent(Key.F5, CONTROL),
    "\x1b[17;5~": KeyPressEvent(Key.F6, CONTROL),
    "\x1b[18;5~": KeyPressEvent(Key.F7, CONTROL),
    "\x1b[19;5~": KeyPressEvent(Key.F8, CONTROL),
    "\x1b[20;5~": KeyPressEvent(Key.F9, CONTROL),
    "\x1b[21;5~": KeyPressEvent(Key.F10, CONTROL),
    "\x1b[23;5~": KeyPressEvent(Key.F11, CONTROL),
    "\x1b[24;5~": KeyPressEvent(Key.F12, CONTROL),
    "\x1b[1;6P": KeyPressEvent(Key.F13, CONTROL),
    "\x1b[1;6Q": KeyPressEvent(Key.F14, CONTROL),
    "\x1b[1;6R": KeyPressEvent(Key.F15, CONTROL),
    "\x1b[1;6S": KeyPressEvent(Key.F16, CONTROL),
    "\x1b[15;6~": KeyPressEvent(Key.F17, CONTROL),
    "\x1b[17;6~": KeyPressEvent(Key.F18, CONTROL),
    "\x1b[18;6~": KeyPressEvent(Key.F19, CONTROL),
    "\x1b[19;6~": KeyPressEvent(Key.F20, CONTROL),
    "\x1b[20;6~": KeyPressEvent(Key.F21, CONTROL),
    "\x1b[21;6~": KeyPressEvent(Key.F22, CONTROL),
    "\x1b[23;6~": KeyPressEvent(Key.F23, CONTROL),
    "\x1b[24;6~": KeyPressEvent(Key.F24, CONTROL),
    "\x1b[62~": KeyPressEvent(Key.ScrollUp, NO_MODS),
    "\x1b[63~": KeyPressEvent(Key.ScrollDown, NO_MODS),
    "\x1b[200~": Key.BracketedPaste,
    "\x1b[E": Key.Ignore,
    "\x1b[G": Key.Ignore,
    "\x1b[3;2~": KeyPressEvent(Key.Delete, SHIFT),
    "\x1b[5;2~": KeyPressEvent(Key.PageUp, SHIFT),
    "\x1b[6;2~": KeyPressEvent(Key.PageDown, SHIFT),
    "\x1b[2;3~": KeyPressEvent(Key.Insert, ALT),
    "\x1b[3;3~": KeyPressEvent(Key.Delete, ALT),
    "\x1b[5;3~": KeyPressEvent(Key.PageUp, ALT),
    "\x1b[6;3~": KeyPressEvent(Key.PageDown, ALT),
    "\x1b[2;4~": KeyPressEvent(Key.Insert, ALT_SHIFT),
    "\x1b[3;4~": KeyPressEvent(Key.Delete, ALT_SHIFT),
    "\x1b[5;4~": KeyPressEvent(Key.PageUp, ALT_SHIFT),
    "\x1b[6;4~": KeyPressEvent(Key.PageDown, ALT_SHIFT),
    "\x1b[3;5~": KeyPressEvent(Key.Delete, CONTROL),
    "\x1b[5;5~": KeyPressEvent(Key.PageUp, CONTROL),
    "\x1b[6;5~": KeyPressEvent(Key.PageDown, CONTROL),
    "\x1b[3;6~": KeyPressEvent(Key.Delete, CONTROL_SHIFT),
    "\x1b[5;6~": KeyPressEvent(Key.PageUp, CONTROL_SHIFT),
    "\x1b[6;6~": KeyPressEvent(Key.PageDown, CONTROL_SHIFT),
    "\x1b[2;7~": KeyPressEvent(Key.Insert, ALT_CONTROL),
    "\x1b[5;7~": KeyPressEvent(Key.PageUp, ALT_CONTROL),
    "\x1b[6;7~": KeyPressEvent(Key.PageDown, ALT_CONTROL),
    "\x1b[2;8~": KeyPressEvent(Key.Insert, ALT_CONTROL_SHIFT),
    "\x1b[5;8~": KeyPressEvent(Key.PageUp, ALT_CONTROL_SHIFT),
    "\x1b[6;8~": KeyPressEvent(Key.PageDown, ALT_CONTROL_SHIFT),
    "\x1b[A": KeyPressEvent(Key.Up, NO_MODS),
    "\x1b[B": KeyPressEvent(Key.Down, NO_MODS),
    "\x1b[C": KeyPressEvent(Key.Right, NO_MODS),
    "\x1b[D": KeyPressEvent(Key.Left, NO_MODS),
    "\x1b[F": KeyPressEvent(Key.End, NO_MODS),
    "\x1b[H": KeyPressEvent(Key.Home, NO_MODS),
    "\x1bOA": KeyPressEvent(Key.Up, NO_MODS),
    "\x1bOB": KeyPressEvent(Key.Down, NO_MODS),
    "\x1bOC": KeyPressEvent(Key.Right, NO_MODS),
    "\x1bOD": KeyPressEvent(Key.Left, NO_MODS),
    "\x1bOF": KeyPressEvent(Key.End, NO_MODS),
    "\x1bOH": KeyPressEvent(Key.Home, NO_MODS),
    "\x1b[1;2A": KeyPressEvent(Key.Up, SHIFT),
    "\x1b[1;2B": KeyPressEvent(Key.Down, SHIFT),
    "\x1b[1;2C": KeyPressEvent(Key.Right, SHIFT),
    "\x1b[1;2D": KeyPressEvent(Key.Left, SHIFT),
    "\x1b[1;2F": KeyPressEvent(Key.End, SHIFT),
    "\x1b[1;2H": KeyPressEvent(Key.Home, SHIFT),
    "\x1b[1;3A": KeyPressEvent(Key.Up, ALT),
    "\x1b[1;3B": KeyPressEvent(Key.Down, ALT),
    "\x1b[1;3C": KeyPressEvent(Key.Right, ALT),
    "\x1b[1;3D": KeyPressEvent(Key.Left, ALT),
    "\x1b[1;3F": KeyPressEvent(Key.End, ALT),
    "\x1b[1;3H": KeyPressEvent(Key.Home, ALT),
    "\x1b[1;4A": KeyPressEvent(Key.Up, ALT_SHIFT),
    "\x1b[1;4B": KeyPressEvent(Key.Down, ALT_SHIFT),
    "\x1b[1;4C": KeyPressEvent(Key.Right, ALT_SHIFT),
    "\x1b[1;4D": KeyPressEvent(Key.Left, ALT_SHIFT),
    "\x1b[1;4F": KeyPressEvent(Key.End, ALT_SHIFT),
    "\x1b[1;4H": KeyPressEvent(Key.Home, ALT_SHIFT),
    "\x1b[1;5A": KeyPressEvent(Key.Up, CONTROL),
    "\x1b[1;5B": KeyPressEvent(Key.Down, CONTROL),
    "\x1b[1;5C": KeyPressEvent(Key.Right, CONTROL),
    "\x1b[1;5D": KeyPressEvent(Key.Left, CONTROL),
    "\x1b[1;5F": KeyPressEvent(Key.End, CONTROL),
    "\x1b[1;5H": KeyPressEvent(Key.Home, CONTROL),
    "\x1b[5A": KeyPressEvent(Key.Up, CONTROL),
    "\x1b[5B": KeyPressEvent(Key.Down, CONTROL),
    "\x1b[5C": KeyPressEvent(Key.Right, CONTROL),
    "\x1b[5D": KeyPressEvent(Key.Left, CONTROL),
    "\x1bOc": KeyPressEvent(Key.Right, CONTROL),
    "\x1bOd": KeyPressEvent(Key.Left, CONTROL),
    "\x1b[1;6A": KeyPressEvent(Key.Up, CONTROL_SHIFT),
    "\x1b[1;6B": KeyPressEvent(Key.Down, CONTROL_SHIFT),
    "\x1b[1;6C": KeyPressEvent(Key.Right, CONTROL_SHIFT),
    "\x1b[1;6D": KeyPressEvent(Key.Left, CONTROL_SHIFT),
    "\x1b[1;6F": KeyPressEvent(Key.End, CONTROL_SHIFT),
    "\x1b[1;6H": KeyPressEvent(Key.Home, CONTROL_SHIFT),
    "\x1b[1;7A": KeyPressEvent(Key.Up, ALT_CONTROL),
    "\x1b[1;7B": KeyPressEvent(Key.Down, ALT_CONTROL),
    "\x1b[1;7C": KeyPressEvent(Key.Right, ALT_CONTROL),
    "\x1b[1;7D": KeyPressEvent(Key.Left, ALT_CONTROL),
    "\x1b[1;7F": KeyPressEvent(Key.End, ALT_CONTROL),
    "\x1b[1;7H": KeyPressEvent(Key.Home, ALT_CONTROL),
    "\x1b[1;8A": KeyPressEvent(Key.Up, ALT_CONTROL_SHIFT),
    "\x1b[1;8B": KeyPressEvent(Key.Down, ALT_CONTROL_SHIFT),
    "\x1b[1;8C": KeyPressEvent(Key.Right, ALT_CONTROL_SHIFT),
    "\x1b[1;8D": KeyPressEvent(Key.Left, ALT_CONTROL_SHIFT),
    "\x1b[1;8F": KeyPressEvent(Key.End, ALT_CONTROL_SHIFT),
    "\x1b[1;8H": KeyPressEvent(Key.Home, ALT_CONTROL_SHIFT),
    "\x1b[1;9A": KeyPressEvent(Key.Up, ALT),
    "\x1b[1;9B": KeyPressEvent(Key.Down, ALT),
    "\x1b[1;9C": KeyPressEvent(Key.Right, ALT),
    "\x1b[1;9D": KeyPressEvent(Key.Left, ALT),
    "\x1b[1;5p": KeyPressEvent("0", CONTROL),
    "\x1b[1;5q": KeyPressEvent("1", CONTROL),
    "\x1b[1;5r": KeyPressEvent("2", CONTROL),
    "\x1b[1;5s": KeyPressEvent("3", CONTROL),
    "\x1b[1;5t": KeyPressEvent("4", CONTROL),
    "\x1b[1;5u": KeyPressEvent("5", CONTROL),
    "\x1b[1;5v": KeyPressEvent("6", CONTROL),
    "\x1b[1;5w": KeyPressEvent("7", CONTROL),
    "\x1b[1;5x": KeyPressEvent("8", CONTROL),
    "\x1b[1;5y": KeyPressEvent("9", CONTROL),
    "\x1b[1;6p": KeyPressEvent("0", CONTROL_SHIFT),
    "\x1b[1;6q": KeyPressEvent("1", CONTROL_SHIFT),
    "\x1b[1;6r": KeyPressEvent("2", CONTROL_SHIFT),
    "\x1b[1;6s": KeyPressEvent("3", CONTROL_SHIFT),
    "\x1b[1;6t": KeyPressEvent("4", CONTROL_SHIFT),
    "\x1b[1;6u": KeyPressEvent("5", CONTROL_SHIFT),
    "\x1b[1;6v": KeyPressEvent("6", CONTROL_SHIFT),
    "\x1b[1;6w": KeyPressEvent("7", CONTROL_SHIFT),
    "\x1b[1;6x": KeyPressEvent("8", CONTROL_SHIFT),
    "\x1b[1;6y": KeyPressEvent("9", CONTROL_SHIFT),
    "\x1b[1;7p": KeyPressEvent("0", ALT_CONTROL),
    "\x1b[1;7q": KeyPressEvent("1", ALT_CONTROL),
    "\x1b[1;7r": KeyPressEvent("2", ALT_CONTROL),
    "\x1b[1;7s": KeyPressEvent("3", ALT_CONTROL),
    "\x1b[1;7t": KeyPressEvent("4", ALT_CONTROL),
    "\x1b[1;7u": KeyPressEvent("5", ALT_CONTROL),
    "\x1b[1;7v": KeyPressEvent("6", ALT_CONTROL),
    "\x1b[1;7w": KeyPressEvent("7", ALT_CONTROL),
    "\x1b[1;7x": KeyPressEvent("8", ALT_CONTROL),
    "\x1b[1;7y": KeyPressEvent("9", ALT_CONTROL),
    "\x1b[1;8p": KeyPressEvent("0", ALT_CONTROL_SHIFT),
    "\x1b[1;8q": KeyPressEvent("1", ALT_CONTROL_SHIFT),
    "\x1b[1;8r": KeyPressEvent("2", ALT_CONTROL_SHIFT),
    "\x1b[1;8s": KeyPressEvent("3", ALT_CONTROL_SHIFT),
    "\x1b[1;8t": KeyPressEvent("4", ALT_CONTROL_SHIFT),
    "\x1b[1;8u": KeyPressEvent("5", ALT_CONTROL_SHIFT),
    "\x1b[1;8v": KeyPressEvent("6", ALT_CONTROL_SHIFT),
    "\x1b[1;8w": KeyPressEvent("7", ALT_CONTROL_SHIFT),
    "\x1b[1;8x": KeyPressEvent("8", ALT_CONTROL_SHIFT),
    "\x1b[1;8y": KeyPressEvent("9", ALT_CONTROL_SHIFT),
    # WSL
    "\x1b[1;3P": KeyPressEvent(Key.F1, ALT),
    "\x1b[1;3Q": KeyPressEvent(Key.F2, ALT),
    "\x1b[1;3R": KeyPressEvent(Key.F3, ALT),
    "\x1b[15;3~": KeyPressEvent(Key.F5, ALT),
    "\x1b[17;3~": KeyPressEvent(Key.F6, ALT),
    "\x1b[18;3~": KeyPressEvent(Key.F7, ALT),
    "\x1b[19;3~": KeyPressEvent(Key.F8, ALT),
    "\x1b[20;3~": KeyPressEvent(Key.F9, ALT),
    "\x1b[21;3~": KeyPressEvent(Key.F10, ALT),
    "\x1b[23;3~": KeyPressEvent(Key.F11, ALT),
    "\x1b[24;3~": KeyPressEvent(Key.F12, ALT),
    "\x1b[1;7P": KeyPressEvent(Key.F1, ALT_CONTROL),
    "\x1b[1;7Q": KeyPressEvent(Key.F2, ALT_CONTROL),
    "\x1b[1;7R": KeyPressEvent(Key.F3, ALT_CONTROL),
    "\x1b[1;7S": KeyPressEvent(Key.F4, ALT_CONTROL),
    "\x1b[15;7~": KeyPressEvent(Key.F5, ALT_CONTROL),
    "\x1b[17;7~": KeyPressEvent(Key.F6, ALT_CONTROL),
    "\x1b[18;7~": KeyPressEvent(Key.F7, ALT_CONTROL),
    "\x1b[19;7~": KeyPressEvent(Key.F8, ALT_CONTROL),
    "\x1b[20;7~": KeyPressEvent(Key.F9, ALT_CONTROL),
    "\x1b[21;7~": KeyPressEvent(Key.F10, ALT_CONTROL),
    "\x1b[23;7~": KeyPressEvent(Key.F11, ALT_CONTROL),
    "\x1b[24;7~": KeyPressEvent(Key.F12, ALT_CONTROL),
    "\x1b[1;4P": KeyPressEvent(Key.F1, ALT_SHIFT),
    "\x1b[1;4Q": KeyPressEvent(Key.F2, ALT_SHIFT),
    "\x1b[1;4R": KeyPressEvent(Key.F3, ALT_SHIFT),
    "\x1b[1;4S": KeyPressEvent(Key.F4, ALT_SHIFT),
    "\x1b[15;4~": KeyPressEvent(Key.F5, ALT_SHIFT),
    "\x1b[17;4~": KeyPressEvent(Key.F6, ALT_SHIFT),
    "\x1b[18;4~": KeyPressEvent(Key.F7, ALT_SHIFT),
    "\x1b[19;4~": KeyPressEvent(Key.F8, ALT_SHIFT),
    "\x1b[20;4~": KeyPressEvent(Key.F9, ALT_SHIFT),
    "\x1b[21;4~": KeyPressEvent(Key.F10, ALT_SHIFT),
    "\x1b[23;4~": KeyPressEvent(Key.F11, ALT_SHIFT),
    "\x1b[24;4~": KeyPressEvent(Key.F12, ALT_SHIFT),
    "\x1b[1;8P": KeyPressEvent(Key.F1, ALT_CONTROL_SHIFT),
    "\x1b[1;8Q": KeyPressEvent(Key.F2, ALT_CONTROL_SHIFT),
    "\x1b[1;8R": KeyPressEvent(Key.F3, ALT_CONTROL_SHIFT),
    "\x1b[1;8S": KeyPressEvent(Key.F4, ALT_CONTROL_SHIFT),
    "\x1b[15;8~": KeyPressEvent(Key.F5, ALT_CONTROL_SHIFT),
    "\x1b[17;8~": KeyPressEvent(Key.F6, ALT_CONTROL_SHIFT),
    "\x1b[18;8~": KeyPressEvent(Key.F7, ALT_CONTROL_SHIFT),
    "\x1b[19;8~": KeyPressEvent(Key.F8, ALT_CONTROL_SHIFT),
    "\x1b[20;8~": KeyPressEvent(Key.F9, ALT_CONTROL_SHIFT),
    "\x1b[21;8~": KeyPressEvent(Key.F10, ALT_CONTROL_SHIFT),
    "\x1b[23;8~": KeyPressEvent(Key.F11, ALT_CONTROL_SHIFT),
    "\x1b[24;8~": KeyPressEvent(Key.F12, ALT_CONTROL_SHIFT),
    "\x1b\x01": KeyPressEvent("a", ALT_CONTROL),
    "\x1b\x02": KeyPressEvent("b", ALT_CONTROL),
    "\x1b\x03": KeyPressEvent("c", ALT_CONTROL),
    "\x1b\x04": KeyPressEvent("d", ALT_CONTROL),
    "\x1b\x05": KeyPressEvent("e", ALT_CONTROL),
    "\x1b\x06": KeyPressEvent("f", ALT_CONTROL),
    "\x1b\x07": KeyPressEvent("g", ALT_CONTROL),
    "\x1b\x08": KeyPressEvent("h", ALT_CONTROL),
    "\x1b\x09": KeyPressEvent("i", ALT_CONTROL),
    "\x1b\x0a": KeyPressEvent("j", ALT_CONTROL),
    "\x1b\x0b": KeyPressEvent("k", ALT_CONTROL),
    "\x1b\x0c": KeyPressEvent("l", ALT_CONTROL),
    "\x1b\x0d": KeyPressEvent("m", ALT_CONTROL),
    "\x1b\x0e": KeyPressEvent("n", ALT_CONTROL),
    "\x1b\x0f": KeyPressEvent("o", ALT_CONTROL),
    "\x1b\x10": KeyPressEvent("p", ALT_CONTROL),
    "\x1b\x11": KeyPressEvent("q", ALT_CONTROL),
    "\x1b\x12": KeyPressEvent("r", ALT_CONTROL),
    "\x1b\x13": KeyPressEvent("s", ALT_CONTROL),
    "\x1b\x14": KeyPressEvent("t", ALT_CONTROL),
    "\x1b\x15": KeyPressEvent("u", ALT_CONTROL),
    "\x1b\x16": KeyPressEvent("v", ALT_CONTROL),
    "\x1b\x17": KeyPressEvent("w", ALT_CONTROL),
    "\x1b\x18": KeyPressEvent("x", ALT_CONTROL),
    "\x1b\x19": KeyPressEvent("y", ALT_CONTROL),
    "\x1b\x1a": KeyPressEvent("z", ALT_CONTROL),
}
